@@ -28,8 +28,6 @@
 #include "sysincludes.h"
 #include "stream.h"
 #include "mtools.h"
-#include "msdos.h"
-#include "scsi.h"
 #include "floppyd_io.h"
 
 /* ######################################################################## */
@@ -588,8 +586,12 @@ Stream_t *FloppydOpen(struct device *dev,
 	}
 
 	if(floppyd_open(This, mode) < 0) {
-		sprintf(errmsg,
-			"Can't open remote drive: %s", strerror(errno));
+		if(errno == EINTR)
+			sprintf(errmsg,
+				"Can't open remote drive: Device in use");
+		else
+			sprintf(errmsg,
+				"Can't open remote drive: %s", strerror(errno));
 		close(This->fd);
 		Free(This);
 		return NULL;
